package com.huigou.topsun.sap.acceptanceFormAck.application.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.topsun.sap.acceptanceFormAck.domain.SapAcceptanceFormAck;
import com.huigou.topsun.sap.acceptanceFormAck.application.SapAcceptanceFormAckApplication;
import com.huigou.topsun.util.TestClientHttp;
import com.huigou.util.SDO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
@author YourUserName
*/
@Service
@RequiredArgsConstructor
public class SapAcceptanceFormAckApplicationImpl  implements SapAcceptanceFormAckApplication  {

	private final ObjectMapper objectMapper;

	@PersistenceContext(unitName = "system")
	private EntityManager entityManager;

	@Override
	public ArrayList<Map<String, Object>> findAcceptanceForm(SDO sdo) {
		String execute;
		Map<String, Object> param = sdo.getProperties();
		List<SapAcceptanceFormAck> acceptanceForms = null;
		try {
			execute = TestClientHttp.execute(param, "http://127.0.0.1:8080/topsun_xt_war/sapAcceptanceForm/queryAcceptanceForm.ajax");
			String resultStr = JSONObject.parseObject(execute).get("data").toString();
			acceptanceForms = objectMapper.readValue(resultStr, new TypeReference<List<SapAcceptanceFormAck>>() {});
		} catch (JsonProcessingException e) {
			e.printStackTrace();
			return null;
		} catch (RuntimeException | IOException e ) {
			throw new RuntimeException(e);
		}
		return objectMapper.convertValue(acceptanceForms, new TypeReference<ArrayList<Map<String, Object>>>() {
		});
	}

	@Override
	public String ackAcceptanceForm(SDO sdo) {
		String execute;
		HashMap<String, Object> param = new HashMap<>(22);
		param.put("data", sdo.getProperties());
		try {
			execute = TestClientHttp.execute(param,
					"http://127.0.0.1:8080/topsun_xt_war/sapAcceptanceForm/saveAcceptanceForm.ajax");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		return execute;
	}

	/**
	 * @description 模拟 SAP 接口 (临时)
	 * @param acceptanceForm 查询条件
	 * @return java.lang.String
	 * @author qinzhenguan
	 * @createDate 2024/1/23 11:50
	 */
	@Override
	public String queryAcceptanceForms(SapAcceptanceFormAck acceptanceForm) throws JsonProcessingException {

		CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
		CriteriaQuery<SapAcceptanceFormAck> query = criteriaBuilder.createQuery(SapAcceptanceFormAck.class);

		Root<SapAcceptanceFormAck> root = query.from(SapAcceptanceFormAck.class);
		List<Predicate> predicates = new ArrayList<>();

		// ======================  设置查询条件  ======================
		settingCriteriaAck(acceptanceForm, predicates, criteriaBuilder, root);

		query.where(criteriaBuilder.and(predicates.toArray(new Predicate[0])));
		List<SapAcceptanceFormAck> results = entityManager.createQuery(query).getResultList();

		return results.size() > 0 ? objectMapper.writeValueAsString(results) : "";
	}

	/***
	 * @description 设置查询条件
	 * @author qinzhenguan
	 * @createDate 2024/1/24 8:22
	 */
	public void settingCriteriaAck(SapAcceptanceFormAck acceptanceForm, List<Predicate> predicates,
								   CriteriaBuilder criteriaBuilder, Root<SapAcceptanceFormAck> root) {
		if(acceptanceForm.getKunag() != null && !acceptanceForm.getKunag().isEmpty()) {
			predicates.add(criteriaBuilder.equal(root.get("kunag"), acceptanceForm.getKunag()));
		}
		if(acceptanceForm.getVkorg() != null && !acceptanceForm.getVkorg().isEmpty()) {
			predicates.add(criteriaBuilder.equal(root.get("vkorg"), acceptanceForm.getVkorg()));
		}
		if(acceptanceForm.getLfdat() != null) {
			predicates.add(criteriaBuilder.equal(root.get("lfdat"), acceptanceForm.getLfdat()));
		}
	}

}
